#!/usr/bin/env node
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/prompts.ts
var import_prompts = __toESM(require("prompts"), 1);
var PROMPT_QUESTIONS = [
  {
    type: "text",
    name: "projectName",
    message: "What should the project be called?"
  },
  {
    type: "select",
    name: "ci",
    message: "Which CI would you like to use?",
    choices: [
      { title: "GitHub Actions", description: "Run your CI with GitHub actions.", value: "github-actions" },
      { title: "No CI", description: "Scaffold a project without any CI pipeline", value: "none" }
    ],
    initial: 0
  }
];
var onCancel = () => {
  console.info("Aborting execution, bye bye \u{1F44B}");
  process.exit();
};
var getUserInput = async function() {
  const response = await (0, import_prompts.default)(PROMPT_QUESTIONS, { onCancel });
  return {
    projectName: response.projectName,
    server: response.server,
    ci: response.ci
  };
};

// src/messages.ts
var import_chalk = __toESM(require("chalk"), 1);
var generateDivider = () => {
  console.info("--------------------------------------------------------------------------------------");
};
var generateBlankLine = () => {
  console.info("");
};
var generateWelcomeMessage = () => {
  generateDivider();
  console.info("                     _                              _   _____                         \n  ___ _ __ ___  __ _| |_ ___       _ __  _   ___  _| |_|___ /        __ _ _ __  _ __  \n / __| '__/ _ \\/ _` | __/ _ \\_____| '_ \\| | | \\ \\/ / __| |_ \\ _____ / _` | '_ \\| '_ \\ \n| (__| | |  __/ (_| | ||  __/_____| | | | |_| |>  <| |_ ___) |_____| (_| | |_) | |_) |\n \\___|_|  \\___|\\__,_|\\__\\___|     |_| |_|\\__,_/_/\\_\\\\__|____/       \\__,_| .__/| .__/ \n                                                                         |_|   |_|    ");
  console.log("                         By sidebase \u219D https://sidebase.io");
  generateDivider();
  generateBlankLine();
  console.info(import_chalk.default.yellowBright("WARNING: create-nuxt3-app is in early development."));
  console.info(import_chalk.default.italic("Initially it uses https://github.com/sidebase/sidebase as a nuxt 3 fullstack baseline,\nbut should soon support more different configurations."));
  generateBlankLine();
};
var generateSuccessMessage = (projectName) => {
  generateBlankLine();
  console.log(import_chalk.default.bold("\u{1F389} Your project has been successfully created!"));
  generateBlankLine();
  console.log("Next steps:");
  console.log(import_chalk.default.italic(`  \u21AA cd ${projectName}/`));
  console.log(import_chalk.default.italic("  \u21AA npm install"));
  console.log(import_chalk.default.italic("  \u21AA npm run dev"));
};

// src/index.ts
var import_node_plop = __toESM(require("node-plop"), 1);

// src/plop.ts
var import_url = __toESM(require("url"), 1);

// src/actions.ts
var import_fs_extra = __toESM(require("fs-extra"), 1);
var import_defu = __toESM(require("defu"), 1);
var addActions = function(plop) {
  plop.setActionType("copy", (answers, config, plop2) => {
    const srcDir = plop2.renderString(config.src, answers);
    const destDir = plop2.renderString(config.dest, answers);
    import_fs_extra.default.copySync(srcDir, destDir, { overwrite: false });
    return "success";
  });
  plop.setActionType("mergeJSON", async (answers, config, plop2) => {
    const primaryJSONFile = plop2.renderString(config.primaryJSONFile, answers);
    const secondaryJSONFile = plop2.renderString(config.secondaryJSONFile, answers);
    const primaryJSON = await import_fs_extra.default.readJson(primaryJSONFile);
    const secondaryJSON = await import_fs_extra.default.readJson(secondaryJSONFile);
    const combinedJSON = (0, import_defu.default)(primaryJSON, secondaryJSON);
    await import_fs_extra.default.writeJson(config.dest, combinedJSON);
    return "success";
  });
  return plop;
};
var actions_default = addActions;

// src/plop.ts
var import_meta = {};
var templateRoot = import_url.default.fileURLToPath(new URL("../templates", import_meta.url));
var addTemplates = function(basePlop) {
  const plop = actions_default(basePlop);
  plop.setGenerator("sidebase", {
    description: "Scaffold a new sidebase project",
    prompts: [
      {
        type: "input",
        name: "projectPath",
        message: "Where should the project be created?"
      },
      {
        type: "confirm",
        name: "server",
        message: "Would you like to use a server?",
        default: false
      },
      {
        type: "list",
        name: "ci",
        message: "Which CI would you like to use?",
        choices: ["github-actions", "none"]
      }
    ],
    actions: (data) => {
      let actions = [];
      actions = actions.concat(
        {
          type: "copy",
          src: `${templateRoot}/sidebase-main/`,
          dest: `${data == null ? void 0 : data.projectPath}/`
        }
      );
      if (data == null ? void 0 : data.server) {
        actions = actions.concat(
          {
            type: "copy",
            src: `${templateRoot}/with-server/`,
            dest: `${data == null ? void 0 : data.projectPath}/`
          },
          {
            type: "mergeJSON",
            primaryJSONFile: `${templateRoot}/sidebase-main/package.json`,
            secondaryJSONFile: `${templateRoot}/with-server/package.json`,
            dest: `${data == null ? void 0 : data.projectPath}/package.json`
          }
        );
      }
      if ((data == null ? void 0 : data.ci) === "github-actions") {
        actions = actions.concat({
          type: "add",
          path: "{{projectPath}}/.github/workflows/ci.yaml",
          templateFile: `${templateRoot}/github-actions/.github/workflows/ci.yaml`,
          abortOnFail: false
        });
      }
      return actions;
    }
  });
  return plop;
};
var plop_default = addTemplates;

// src/index.ts
(async () => {
  generateWelcomeMessage();
  const answers = await getUserInput();
  const plop = await (0, import_node_plop.default)();
  const plopWithTemplates = plop_default(plop);
  const sidebaseGenerator = plopWithTemplates.getGenerator("sidebase");
  const projectPath = `${process.cwd()}/${answers.projectName}`;
  sidebaseGenerator.runActions({ projectPath, ci: answers.ci, server: false }).then(function(results) {
    if (results.failures.length !== 0) {
      console.error("An error has occurred while scaffolding your project.");
      return console.log("Errors:", results.failures);
    }
    generateSuccessMessage(answers.projectName);
  });
})();
